package cn.aibreeze.exam.api;

import cn.aibreeze.exam.dto.SessionExamResult;
import cn.aibreeze.exam.dto.SingleQaResult;
import cn.aibreeze.exam.entity.ElehKnowledgeDo;
import cn.aibreeze.exam.entity.ElehSessionQaRecordDo;
import cn.aibreeze.exam.mapper.ElehKnowledgeMapper;
import cn.aibreeze.exam.mapper.ElehKnowledgeTagMapper;
import cn.aibreeze.exam.mapper.ElehSessionQaRecordMapper;
import cn.aibreeze.exam.util.ExamConstants;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Controller;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.io.PrintWriter;
import java.nio.charset.Charset;
import java.text.DecimalFormat;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Controller
@Slf4j
@RequestMapping(value = "/exam/h5/")
@RequiredArgsConstructor
public class H5Controller {

    private final ElehKnowledgeMapper knowledgeMapper;

    private final ElehKnowledgeTagMapper tagMapper;

    private final ElehSessionQaRecordMapper qaRecordMapper;


    @SneakyThrows
    @GetMapping(value = "exam_result/{tenant}/{session_id}/{user_id}.html")
    public void list(@PathVariable("tenant") Integer tenant,
                     @PathVariable("session_id") String sessionId,
                     @PathVariable("user_id") String userId,
                     HttpServletResponse response) {


        response.setContentType("text/html");
        response.setCharacterEncoding("UTF-8");
        PrintWriter out = response.getWriter();
        out.println("<!DOCTYPE html>\n" +
                "<html lang=\"zh\">\n" +
                "<head>\n" +
                "    <meta charset=\"UTF-8\">\n" +
                "    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0\">\n" +
                "    <title>考试结果</title>\n" +
                "    <style>\n" +
                "        body {\n" +
                "            font-family: Arial, sans-serif;\n" +
                "            margin: 0;\n" +
                "            padding: 0;\n" +
                "        }\n" +
                "\n" +
                "        header {\n" +
                "            background-color: #f1f1f1;\n" +
                "            text-align: center;\n" +
                "            padding: 20px;\n" +
                "        }\n" +
                "\n" +
                "        main {\n" +
                "            padding: 20px;\n" +
                "        }\n" +
                "    </style>\n" +
                "</head>");
        out.println("<body>");

        // 查询考试结果
        LambdaQueryWrapper<ElehSessionQaRecordDo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ElehSessionQaRecordDo::getTenant, tenant);
        queryWrapper.eq(ElehSessionQaRecordDo::getSessionId, sessionId);
        queryWrapper.eq(ElehSessionQaRecordDo::getUserId, userId);
        List<ElehSessionQaRecordDo> qaRecordDos = qaRecordMapper.selectList(queryWrapper);
        if(!CollectionUtils.isEmpty(qaRecordDos)) {

            List<ElehSessionQaRecordDo> userQaList = qaRecordDos.stream().filter(v -> Objects.nonNull(v.getScore())).
                    filter(v -> Objects.equals(v.getType(), ExamConstants.QA_RESULT_TYPE_SINGLE_QUESTION)).collect(Collectors.toList());

            Double totalScore = userQaList.stream().mapToDouble(ElehSessionQaRecordDo::getScore).sum();
            long examNum = userQaList.stream().count();
            DecimalFormat df = new DecimalFormat("#.##");
            String avgScore = df.format(totalScore / examNum);

            out.println("<main>");
            out.println("<p><strong>恭喜你，考试结束了。</strong></p>");
            out.println("<p><strong>你一共考了</strong> "+examNum+" <strong>道题目</strong></p>");
            out.println("<p><strong>最终得分：</strong>"+avgScore+"</p>");

            List<ElehSessionQaRecordDo> examResult = qaRecordDos.stream().filter(v -> Objects.equals(v.getType(), ExamConstants.QA_RESULT_TYPE_SINGLE_EXAM)).collect(Collectors.toList());
            if(!CollectionUtils.isEmpty(examResult)) {
                String examCriteria = examResult.get(0).getExamCriteria();
                if(StringUtils.isNotEmpty(examCriteria)) {
                    out.println("<p><strong>考试点评：</strong>"+examCriteria+"</p>");
                }
            }

            out.println("<br/>");

            // 数据转换
            userQaList.stream().forEach(v -> {

                ElehKnowledgeDo knowledgeDo = knowledgeMapper.selectById(v.getTenant(), v.getQuestionId());

                SingleQaResult qa = new SingleQaResult();
                qa.setQuestion(knowledgeDo.getQuestion());
                qa.setAnswer(knowledgeDo.getAnswer());
                qa.setUserAnswer(v.getAnswer());
                qa.setScore(v.getScore());
                qa.setScoreCriteria(v.getScoreCriteria());


                out.println("<p><strong>考试题目：</strong>"+knowledgeDo.getQuestion()+"</p>");
                out.println("<p><strong>你的得分：</strong>"+getOrEmpty(v.getScore())+"</p>");
                out.println("<p><strong>参考答案：</strong>"+knowledgeDo.getAnswer()+"</p>");
                out.println("<p><strong>你的答案：</strong>"+getOrEmpty(v.getAnswer())+"</p>");
                out.println("<p><strong>得分点评：</strong>"+getOrEmpty(v.getScoreCriteria())+"</p>");

                out.println("<br/>");
            });

            out.println("</main>");
        } else {
            out.println("<header>\n"
                    +"   <h1>你还没有任何考试信息哦</h1>\n"
                    +"    </header>");
        }
        out.println("</body>");
        out.println("</html>");

    }

    private String getOrEmpty(Object str) {
        str = Objects.isNull(str) ? "" : str;
        return str.toString();
    }
}
